import numpy as np
from scipy import stats
from ..base_indicator import BaseIndicator
from indicators.common.common import CommonMethods


class POLY_TREND(BaseIndicator):
    def __init__(self):
        super().__init__("Poly Trend")

    def calculate(self, var_num, param1, param2, open_prices, high, low, close):
        #var_num is either 1/2/3 for linear/quadratic/cubic trend
        open_prices, high, low, close = map(np.array, (open_prices, high, low, close))
        n = len(close)
        
        lookback = int(param1 + 0.5)    # Lookback for trend
        atr_length = int(param2 + 0.5)  # Lookback for ATR normalization
        front_bad = max(lookback-1, atr_length)
        front_bad = min(front_bad, n)

        work1, work2, work3 = CommonMethods.legendre_3(lookback)
        output = np.zeros(n)

        log_close = np.log(close)
        atr_values = np.concatenate([np.zeros(atr_length-1), CommonMethods.atr(open_prices, high, low, close, atr_length)])

        for icase in range(front_bad, n):
            if var_num == "1":
                dptr = work1
            elif var_num == "2":
                dptr = work2
            elif var_num == "3":
                dptr = work3

            window = log_close[icase-lookback+1:icase+1]
            mean = np.mean(window)
            dot_prod = np.dot(window, dptr)

            k = lookback - 1 if lookback != 2 else 2
            denom = atr_values[icase] * k
            output[icase] = dot_prod * 2.0 / (denom + 1e-60)

            # Compute R-square
            yss = np.sum((window - mean)**2)
            pred = dot_prod * dptr
            rsq = 1.0 - np.sum((window - mean - pred)**2) / (yss + 1e-60)
            rsq = max(rsq, 0.0)

            output[icase] *= rsq

            output[icase] = 100.0 * stats.norm.cdf(output[icase]) - 50.0

        return output